"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var DragDropListDirective = (function () {
    function DragDropListDirective(el) {
        this.el = el;
        this.draggable = true;
        this.onDrop = new core_1.EventEmitter();
        this.shiftDown = false;
    }
    DragDropListDirective.prototype.dragstart = function ($event) {
        var dragIndex = this.dragDropList[0].toString();
        $event.dataTransfer.setData('dragIndex', dragIndex);
        this.addDraggedStyle();
    };
    DragDropListDirective.prototype.dragend = function () {
        this.removeDraggedStyle();
    };
    DragDropListDirective.prototype.dragover = function ($event) {
        $event.preventDefault();
        this.addDragOveredStyle();
    };
    DragDropListDirective.prototype.dragleave = function () {
        this.removeDragOveredStyle();
    };
    DragDropListDirective.prototype.drop = function ($event) {
        var dropIndex = this.dragDropList[0];
        var dragIndex = +$event.dataTransfer.getData('dragIndex');
        var array = this.dragDropList[1];
        this.removeDragOveredStyle();
        if (dropIndex !== dragIndex) {
            var temp = array[dragIndex];
            array[dragIndex] = array[dropIndex];
            array[dropIndex] = temp;
            this.onDrop.emit(array);
        }
    };
    DragDropListDirective.prototype.keyup = function ($event) {
        switch ($event.which) {
            case 16:
                this.shiftDown = false;
                this.removeDraggedStyle();
                break;
        }
    };
    DragDropListDirective.prototype.keydown = function ($event) {
        switch ($event.which) {
            case 38:
                this.keydownDown();
                break;
            case 40:
                this.keydownUp();
                break;
            case 16:
                this.shiftDown = true;
                this.addDraggedStyle();
                break;
        }
    };
    DragDropListDirective.prototype.removeDragOveredStyle = function () {
        if (this.draggOvered) {
            this.el.nativeElement.classList.remove(this.draggOvered);
        }
        else {
            this.el.nativeElement.style.outline = 'none';
        }
    };
    DragDropListDirective.prototype.addDragOveredStyle = function () {
        if (this.draggOvered) {
            this.el.nativeElement.classList.add(this.draggOvered);
        }
        else {
            this.el.nativeElement.style.outline = '2px dashed lightblue';
        }
    };
    DragDropListDirective.prototype.addDraggedStyle = function () {
        if (this.dragged) {
            this.el.nativeElement.classList.add(this.dragged);
        }
        else {
            this.el.nativeElement.style.opacity = 0.6;
        }
    };
    DragDropListDirective.prototype.removeDraggedStyle = function () {
        if (this.dragged) {
            this.el.nativeElement.classList.remove(this.dragged);
        }
        else {
            this.el.nativeElement.style.opacity = 1;
        }
    };
    DragDropListDirective.prototype.keydownDown = function () {
        if (this.shiftDown) {
            this.switchPrev();
        }
        else {
            this.focusPrev();
        }
    };
    DragDropListDirective.prototype.keydownUp = function () {
        if (this.shiftDown) {
            this.switchNext();
        }
        else {
            this.focusNext();
        }
    };
    DragDropListDirective.prototype.switchNext = function () {
        var _this = this;
        var dropIndex = this.dragDropList[0];
        var array = this.dragDropList[1];
        var swapIndex, temp;
        swapIndex = (dropIndex + 1) % array.length;
        temp = array[dropIndex];
        array[dropIndex] = array[swapIndex];
        array[swapIndex] = temp;
        setTimeout(function () {
            _this.el.nativeElement.focus();
        }, 0);
    };
    DragDropListDirective.prototype.switchPrev = function () {
        var _this = this;
        var dropIndex = this.dragDropList[0];
        var array = this.dragDropList[1];
        var swapIndex, temp;
        swapIndex = (dropIndex - 1) % array.length;
        if (swapIndex < 0) {
            swapIndex += array.length;
        }
        temp = array[dropIndex];
        array[dropIndex] = array[swapIndex];
        array[swapIndex] = temp;
        setTimeout(function () {
            _this.el.nativeElement.focus();
        }, 0);
    };
    DragDropListDirective.prototype.focusNext = function () {
        var elem = this.el.nativeElement;
        var brothers = elem.parentElement.children;
        var array = [].slice.call(brothers);
        var myIndex = array.indexOf(this.el.nativeElement);
        var nextIndex = (myIndex + 1) % array.length;
        array[nextIndex].focus();
    };
    DragDropListDirective.prototype.focusPrev = function () {
        var elem = this.el.nativeElement;
        var brothers = elem.parentElement.children;
        var array = [].slice.call(brothers);
        var myIndex = array.indexOf(this.el.nativeElement);
        var prevIndex = (myIndex - 1) < 0 ? array.length + (myIndex - 1) : (myIndex - 1);
        array[prevIndex].focus();
    };
    __decorate([
        core_1.Input(),
        __metadata("design:type", Array)
    ], DragDropListDirective.prototype, "dragDropList", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", String)
    ], DragDropListDirective.prototype, "dragged", void 0);
    __decorate([
        core_1.Input(),
        __metadata("design:type", String)
    ], DragDropListDirective.prototype, "draggOvered", void 0);
    __decorate([
        core_1.HostBinding(),
        __metadata("design:type", Object)
    ], DragDropListDirective.prototype, "draggable", void 0);
    __decorate([
        core_1.Output(),
        __metadata("design:type", Object)
    ], DragDropListDirective.prototype, "onDrop", void 0);
    __decorate([
        core_1.HostListener('dragstart', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [DragEvent]),
        __metadata("design:returntype", void 0)
    ], DragDropListDirective.prototype, "dragstart", null);
    __decorate([
        core_1.HostListener('dragend'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], DragDropListDirective.prototype, "dragend", null);
    __decorate([
        core_1.HostListener('dragover', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], DragDropListDirective.prototype, "dragover", null);
    __decorate([
        core_1.HostListener('dragleave'),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], DragDropListDirective.prototype, "dragleave", null);
    __decorate([
        core_1.HostListener('drop', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [DragEvent]),
        __metadata("design:returntype", void 0)
    ], DragDropListDirective.prototype, "drop", null);
    __decorate([
        core_1.HostListener('keyup', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], DragDropListDirective.prototype, "keyup", null);
    __decorate([
        core_1.HostListener('keydown', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], DragDropListDirective.prototype, "keydown", null);
    DragDropListDirective = __decorate([
        core_1.Directive({
            selector: '[dragDropList]'
        }),
        __metadata("design:paramtypes", [core_1.ElementRef])
    ], DragDropListDirective);
    return DragDropListDirective;
}());
exports.DragDropListDirective = DragDropListDirective;
//# sourceMappingURL=drag-drop-list.directive.js.map